
/**
 * AccountBean -- Base Account Bean
 * This class must extend
 * javax.ejb.SessionBean and implement
 * the methods in this interface as well as providing
 * the implementation of the business methods.
 *
 */

package ejbay.account;

//import ejbstubs.*;

import javax.ejb.*;

import java.io.Serializable;
import java.util.*;
import java.rmi.*;
import javax.rmi.PortableRemoteObject;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.net.*;
import javax.naming.*;

import java.rmi.*;
import java.io.*;
import java.util.Properties;
import ejbay.exceptions.*;

public class AccountBean implements javax.ejb.SessionBean {
    static final boolean verbose = false;
    
    public Integer AcctCreate(UserObj user) throws java.rmi.RemoteException,
	InvalidUserInfo, DuplicateAccount{
	
        if (verbose)
            System.out.println("AcctCreate called");
        if(null == user)
            throw new InvalidUserInfo("No user entered");
        
        try 
	{
            if(verbose)
		System.out.println(user);
            Context jndiCntx = new InitialContext();
            Object ref = jndiCntx.lookup("UserEntityLocalHome");
            UserEntityLocalHome home =
		(UserEntityLocalHome)PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);
            UserEntityLocal user_1 = home.create(user);
            
            return user_1.getUserID();
	}
	catch(javax.naming.NamingException ne)
	{
	    if(verbose)
		System.out.println("Server error trying to lookup JNDI context");
	    throw new RemoteException("Could not get JNDI context");
	}
	catch(javax.ejb.CreateException ce) {
	    if(verbose)
		System.out.println("Catching create exception");
            throw new DuplicateAccount("Account " + user.getEmailAdd() + " already exists.");
        }
	catch(Exception e) {
	    if(verbose)
		System.out.println("Catching generic exception");
            throw new RemoteException("Failed to create account");
        }	
    }
    
    public String AcctLogin(String emailAdd, String password) throws java.rmi.RemoteException,
            InvalidUserPass, InvalidUserInfo, ObjectNotFoundException {
        if (verbose)
            System.out.println("AcctLogin called");
        
        try {
            Context jndiCntx = new InitialContext();
            
            Object ref = jndiCntx.lookup("UserEntityLocalHome");
            UserEntityLocalHome home =
                    (UserEntityLocalHome)PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);
            UserEntityLocal user_1 =
                    home.findValidateUser(emailAdd,password);

            user_1.setTransID(new Integer(0));
	    if (verbose)
		System.out.println("Found user: "+user_1.getEmailAdd()+"("+user_1.getUserID()+")");
            
            if(user_1 == null)
                throw new InvalidUserPass("Bad username or password");
            
            return user_1.getUserID().toString();
        } catch(ObjectNotFoundException onf) {
	    if(verbose)
		System.out.println("Object not found!");
            return new String("0");
        } catch(Exception e) {
	    if(verbose)
		System.out.println(e);
        }
        return "1";
    };
    
    public void AcctUpdate(UserObj user, Integer sessionKey) throws java.rmi.RemoteException, InvalidUserInfo, UserNotLoggedIn {
        if (verbose)
            System.out.println("AcctUpdate called");
        
        if (sessionKey == null || sessionKey.equals(new Integer(0)))
	    {
		if(verbose)
		    System.out.println("No user found for sessionKey = " + sessionKey);
	    throw new UserNotLoggedIn("No user found for sessionKey = " + sessionKey);
	}
        if (null == user)
            throw new InvalidUserInfo("No user info given");

        try {
            
            int sKey = sessionKey.intValue();
            
            Context jndiCntx = new InitialContext();
            Object ref = jndiCntx.lookup("UserEntityLocalHome");
            UserEntityLocalHome home = (UserEntityLocalHome)
            PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);
            UserEntityLocal userBean = home.findByPrimaryKey(new Integer(sKey));
            
            if (userBean == null)
                throw new InvalidUserInfo("Invalid session key");
            if(verbose)
		System.out.println("Updating user: " + userBean.toUserObj());

            if ((user.getEmailAdd().equals(" ")) == false)
                userBean.setEmailAdd(user.getEmailAdd());
            if (user.getFirstName().equals(" ") == false)
                userBean.setFirstName(user.getFirstName());
            if (user.getLastName().equals(" ") == false)
                userBean.setLastName(user.getLastName());
            if (user.getPassword().equals(" ") == false)
                userBean.setPassword(user.getPassword());
            if (user.getCredCardName().equals(" ") == false)
                userBean.setCredCardName(user.getCredCardName());
            if (user.getCreditCardNum().equals(" ") == false)
                userBean.setCreditCardNum(user.getCreditCardNum());
            if (user.getCCExpDate().equals(" ") == false)
                userBean.setCCExpDate(user.getCCExpDate());

	    if(verbose)
		System.out.println("To user: " + userBean.toUserObj());
        }
	catch(javax.ejb.FinderException fe)
	{
	    
	    if(verbose) System.out.println("No user found for sessionKey = " + sessionKey);
	    throw new UserNotLoggedIn("No user found for sessionKey = " + sessionKey);
	}
	catch(javax.ejb.EJBException ejbe)
	{
	    if(verbose) System.out.println("User with that email address already exists");
	    throw new InvalidUserInfo("User with that email address already exists");
	}
	catch(InvalidUserInfo iuie)
	{
	    if(verbose) System.out.println("No user found for sessionKey = " + sessionKey);
	    throw new UserNotLoggedIn("No user found for sessionKey = " + sessionKey);
	}
	catch(Exception e) 
	{
            if(verbose) System.out.println(e);
        }
    };
    
    public void AcctLogout(String sessionKey) throws java.rmi.RemoteException, UserNotLoggedIn {
        if (verbose)
            if(verbose) System.out.println("AcctLogout called");
        if (sessionKey == null || sessionKey.equals(""))
            throw new UserNotLoggedIn("No session key given");
    };
    
    /* this function shouldn't exist
    public void AcctDelete(UserObj user) throws java.rmi.RemoteException, InvalidUserPass, InvalidUserInfo, UserNotLoggedIn {
        if (verbose)
            if(verbose) System.out.println("AcctDelete called");
        if (null == user)
            throw new InvalidUserInfo("No user entered");
    }
    */
    

    public UserObj AcctView(String emailAdd)throws java.rmi.RemoteException, InvalidUserInfo, UserNotLoggedIn 
    {		
	if (verbose)
	    if(verbose) System.out.println("AcctView called");
	
	if(null == emailAdd || emailAdd.equals(""))
	    throw new InvalidUserInfo("No user name entered");
	
	try
	{
	    Context jndiCntx = new InitialContext();
	    
	    Object ref = jndiCntx.lookup("UserEntityLocalHome");
	    UserEntityLocalHome home = (UserEntityLocalHome)
		PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);   
	    UserEntityLocal user_1 = home.findByEmailAdd(emailAdd);
	    
	    return user_1.toUserObj();
	}
	catch(javax.ejb.FinderException fe)
	{
	    if(verbose) System.out.println("Could not find user " + emailAdd + " in the database.");
	    throw new InvalidUserInfo("Invalid user information given");
	}
	catch(javax.naming.NamingException ne)
	{
	    if(verbose) System.out.println("Server error trying to lookup JNDI context");
	    throw new RemoteException("Could not get JNDI context");
	}
	catch(Exception e)
	{
	    e.printStackTrace();		
	}
	
	return null;
    }
	
    public UserObj AcctView(int userID) throws java.rmi.RemoteException, InvalidUserInfo, UserNotLoggedIn
    {
	if (verbose)
	    if(verbose) System.out.println("AcctView called");
	
	if(userID < 0)
	    throw new InvalidUserInfo("Invalid user ID specified");
	
	try
	{
	    Context jndiCntx = new InitialContext();
	    
	    Object ref = jndiCntx.lookup("UserEntityLocalHome");
	    UserEntityLocalHome home = (UserEntityLocalHome)
		PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);   
	    UserEntityLocal user_1 = home.findByPrimaryKey(new Integer(userID));
	    
	    return user_1.toUserObj();
	}
	catch(javax.ejb.FinderException fe)
	{
	    if(verbose) System.out.println("Could not find user " + userID + " in the database.");
	    throw new InvalidUserInfo("Invalid user information given");
	}
	catch(javax.naming.NamingException ne)
	{
	    if(verbose) System.out.println("Server error trying to lookup JNDI context");
	    throw new RemoteException("Could not get JNDI context");
	}
	catch(Exception e)
	{
	    e.printStackTrace();		
	}
	
	return null;
    }

    public void isAlive() throws java.rmi.RemoteException
    {}

    public void setSessionContext( SessionContext context ) {
        this.context = context;
    };
    
    public void ejbCreate() {}
    
    protected SessionContext context;
    
    public void ejbActivate() {}
    public void ejbPassivate() {}
    public void ejbRemove() {}
    
}
