package ejbay.account;

import java.io.Serializable;
import java.lang.IllegalArgumentException;

/**
 * ideas loosly borrowed from 2005 team 1 Account.java
 * @author jtrapass 
 */

public class UserObj implements java.io.Serializable {
    /**
     *Matches this sql database stuff:
     *
     *mysql> describe Users;
     *+---------------+--------------+------+-----+---------+----------------+
     *| Field         | Type         | Null | Key | Default | Extra          |
     *+---------------+--------------+------+-----+---------+----------------+
     *| UserID        | int(11)      | NO   | PRI | NULL    | auto_increment |
     *| EmailAdd      | varchar(100) | NO   |     |         |                |
     *| FirstName     | varchar(100) | NO   |     |         |                |
     *| LastName      | varchar(100) | NO   |     |         |                |
     *| Password      | varchar(20)  | NO   |     |         |                |
     *| CredCardName  | varchar(200) | YES  |     | NULL    |                |
     *| CreditCardNum | varchar(20)  | YES  |     | NULL    |                |
     *| CCExpDate     | varchar(10)  | YES  |     | NULL    |                |
     *+---------------+--------------+------+-----+---------+----------------+
     */

    /**
     * UserID is autogenerated by the database, so don't create it outselves,
     * however, there must be an infrastructure for accepting it since we do
     * queries, etc
     */
    private int UserID;
    private String EmailAdd;
    private String FirstName;
    private String LastName;
    private String Password;
    private String CredCardName;
    private String CreditCardNum;
    private String CCExpDate;
    
    /**
     * base creation format
     *   
     * @param emailAdd not permitted to be null or \"\"
     * @param emailAdd not permitted to be null or \"\"
     * @param firstName not permitted to be null or \"\"
     * @param lastName not permitted to be null or \"\"
     * @param password not permitted to be null or \"\"
     * @param credCardName if null creditCardNum and ccExpDate must be as well
     * @param creditCardNum if null credCardName and ccExpDate must be as well
     * @param ccExpDate if null creditCardNum and credCardName must be as well
     */
    public UserObj(String emailAdd,
		   String firstName,
		   String lastName,
		   String password,
		   String credCardName,
		   String creditCardNum,
		   String ccExpDate) 
	throws IllegalArgumentException
    {
	
	/*consistancy checking -- pulled causes problems in multiphase processing
	if(emailAdd == null || emailAdd.replaceAll("[\r\n]", "").equals(""))
	    throw new IllegalArgumentException("emailAdd");
	if(firstName == null || firstName.replaceAll("[\r\n]", "").equals(""))
	    throw new IllegalArgumentException("firstName");
	if(lastName == null || lastName.replaceAll("[\r\n]", "").equals(""))
	    throw new IllegalArgumentException("lastName");
	if(password == null || password.replaceAll("[\r\n]", "").equals(""))
	    throw new IllegalArgumentException("password");
	
	if(credCardName == null || credCardName.replaceAll("[\r\n]", "").equals("") ||
	   creditCardNum == null || creditCardNum.replaceAll("[\r\n]", "").equals("") ||
	   ccExpDate == null || ccExpDate.replaceAll("[\r\n]", "").equals("")){
	    if(credCardName != null && !credCardName.replaceAll("[\r\n]", "").equals(""))
		throw new IllegalArgumentException("credit card info");
	    if(creditCardNum != null && !creditCardNum.replaceAll("[\r\n]", "").equals(""))
		throw new IllegalArgumentException("credit card info");
	    if(ccExpDate != null && !ccExpDate.replaceAll("[\r\n]", "").equals(""))
		throw new IllegalArgumentException("credit card info");
	    
	    credCardName = null;
	    creditCardNum = null;
	    ccExpDate = null;
	}
	
	*/

	//place holder for value not yet placed in database
	UserID = -1;
	
	EmailAdd = emailAdd;
	FirstName = firstName;
	LastName = lastName;
	Password = password;
	CredCardName = credCardName;
	CreditCardNum = creditCardNum;
	CCExpDate = ccExpDate;
    }

    /*public UserObj(String emailAdd,
		   String password) 
	throws IllegalArgumentException
    {
	/*consistancy checking
	if(emailAdd == null || emailAdd.replaceAll("[\r\n]", "").equals(""))
	    throw new IllegalArgumentException("emailAdd");
	if(password == null || password.replaceAll("[\r\n]", "").equals(""))
	    throw new IllegalArgumentException("password");

            EmailAdd = emailAdd;
            Password = password;

	}

   */


    public UserObj(String emailAdd,
		   String firstName,
		   String lastName,
		   String password)
	throws IllegalArgumentException
    {
	this(emailAdd,firstName,lastName,password,null,null,null);
    }

    public String toString(){
	return 
	    "[UserID = " + UserID +
	    ", EmailAdd = " + EmailAdd +
	    ", FirstName = " + FirstName +
	    ", LastName = " + LastName +
	    ", Password = " + Password +
	    ", CredCardName = " + (CredCardName==null?"NULL":CredCardName) +
	    ", CreditCardNum = " + (CreditCardNum==null?"NULL":CreditCardNum) +
	    ", CCExpDate = " + (CCExpDate==null?"NULL":CCExpDate) + "]";
    }

    public void setUserID(int userID){
	UserID = userID;
    }
    public int getUserID(){
	return UserID;
    }
    
    public void setEmailAdd(String emailAdd){
	EmailAdd = emailAdd;
    }
    public String getEmailAdd(){
	return EmailAdd;
    }

    public void setFirstName(String firstName){
	FirstName = firstName;
    }
    public String getFirstName(){
	return FirstName;
    }

    public void setLastName(String lastName){
	LastName = lastName;
    }
    public String getLastName(){
	return LastName;
    }

    public void setPassword(String password){
	Password = password;
    }
    public String getPassword(){
	return Password;
    }

    public void setCredCardName(String credCardName){
	CredCardName = credCardName;
    }
    public String getCredCardName(){
	return CredCardName;
    }

    public void setCreditCardNum(String creditCardNum){
	CreditCardNum = creditCardNum;
    }
    public String getCreditCardNum(){
	return CreditCardNum;
    }

    public void setCCExpDate(String ccExpDate){
	CCExpDate = ccExpDate;
    }
    public String getCCExpDate(){
	return CCExpDate;
    }
    
    public boolean equals(UserObj newUser) {
        if ((newUser.getUserID() == this.UserID) &&
            (newUser.getEmailAdd()).equals(this.EmailAdd) &&
            (newUser.getFirstName()).equals(this.FirstName) &&
            (newUser.getLastName()).equals(this.LastName) &&
            (newUser.getPassword()).equals(this.Password) &&
            (newUser.getCredCardName()).equals(this.CredCardName) &&
            (newUser.getCreditCardNum()).equals(this.CreditCardNum) &&
            (newUser.getCCExpDate()).equals(this.CCExpDate))
            return true;
        return false;    
    }
}
