
   /**
    * AuctionBean -- Base Auction Bean
    * This class must extend
    * javax.ejb.SessionBean and implement
    * the methods in this interface.
    *
    */

package ejbay.auction;

//import ejbstubs.*;

import javax.ejb.*;

import java.io.Serializable;
import java.util.*;
import java.rmi.*;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.net.*;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;

import javax.naming.*;
import javax.rmi.PortableRemoteObject;

import ejbay.exceptions.*;
import ejbay.account.*;
import ejbay.auction.*;
import ejbay.*;

import java.lang.reflect.Array;

public class AuctionBean implements javax.ejb.SessionBean
{  
    static final boolean verbose = false;
    
    public AuctionObj AuctionPost(Integer sessionKey ,AuctionObj aucObj, Integer aucLen, Integer TID) throws java.rmi.RemoteException, UserNotLoggedIn, InvalidAuction{
	if (verbose)
	    System.out.println("AuctionPost called");

	if(null == aucObj)
	    throw new InvalidAuction("No auction information given");

	try {
	    Context jndiCntx = new InitialContext();

	    //check if this auction is already posted
	    Object Uref = jndiCntx.lookup("UserEntityLocalHome");
	    UserEntityLocalHome Uhome = (UserEntityLocalHome)PortableRemoteObject.narrow(Uref,UserEntityLocalHome.class);
	    UserEntityLocal user_1 = Uhome.findByPrimaryKey(sessionKey);

	    if(user_1.getTransID().equals(TID)){
		// we need to find the bid/auction that was posted previously

		if (verbose) System.out.println("Auction already exists in database, based on transID... find it!");
		
		return null;  //jtrapass TODO: THIS IS WRONG!!!
	    }else{
		String sqlQuery = "SELECT NOW(), DATE_ADD(NOW(),INTERVAL "+aucLen.toString()+" DAY)";

		if (verbose) System.out.println("stuff "+sqlQuery);
		ResultSet rs = DatabaseAccessor.query(sqlQuery);
		
		if(rs.next()) {	    
		    aucObj.setStartDate(rs.getString(1));
		    aucObj.setEndDate(rs.getString(2));
		    if (verbose) System.out.println("Now: "+rs.getString(1)+" End: "+rs.getString(2));
		}
		aucObj.setAuctionID(new Integer(2));
		if (verbose) System.out.println("Start Bid: " + aucObj.getStartBid());
		if (verbose) System.out.println("Creating Auction:");
		if (verbose) System.out.println(aucObj.toString());
		
		Object ref = jndiCntx.lookup("AuctionEntityLocalHome");
		AuctionEntityLocalHome home = (AuctionEntityLocalHome)PortableRemoteObject.narrow(ref,AuctionEntityLocalHome.class);
		AuctionEntityLocal auc_1 = home.create(aucObj, TID);
		
		if (verbose) System.out.println("Created a new auction with ID "+auc_1.getAuctionID());
		
		return auc_1.toAuctionObj();
	    }
	}catch(CreateException ce) {
	    if (verbose) System.out.println("Unable to create new auction");
	    throw new InvalidAuction("Unable to create new auction");
	} catch(NamingException ne) {
	    if (verbose) System.out.println("Error during JNDI lookup of beans.");
	    throw new InvalidAuction("Unable to create new auction, JNDI problems");
	} catch(Exception e) {
	    e.printStackTrace();
	    throw new InvalidAuction("Unable to create new auction...?");
	}
	
	//return null;
    };
    
    public AuctionObj AuctionView(int auctionID)throws java.rmi.RemoteException, InvalidAuction
    {
	if (verbose)
	    if (verbose) System.out.println("AuctionView called");
	
	if(auctionID < 0)
	    throw new InvalidAuction("AuctionID cannot be less than zero");
	try
	{
	    Context jndiCntx = new InitialContext();
	    
	    Object ref = jndiCntx.lookup("AuctionEntityLocalHome");
	    AuctionEntityLocalHome home = (AuctionEntityLocalHome)
		PortableRemoteObject.narrow(ref,AuctionEntityLocalHome.class);   
	    AuctionEntityLocal auc_1 = home.findByPrimaryKey(new Integer(auctionID));

	    //	    if (verbose) System.out.println("auction ID = " + auc_1.toAuctionObj().getAuctionID());

	    return validateAuctionData(auc_1.toAuctionObj());
	}
	catch(Exception e){if (verbose) System.out.println(e);}
	
	return null;
    };
   
    public AuctionObj[] AuctionSearch(String name) throws java.rmi.RemoteException, InvalidAuction{
        
	if (name == null)
            throw new InvalidAuction("No name to search on.");
	if (verbose)
            if (verbose) System.out.println("AuctionSearch("+name+") called");

	try {
	    Context jndiCntx = new InitialContext();

	    Object ref = jndiCntx.lookup("AuctionEntityLocalHome");
	    AuctionEntityLocalHome home = (AuctionEntityLocalHome)
	    PortableRemoteObject.narrow(ref,AuctionEntityLocalHome.class);
	    
	    Collection aucCollect = home.findAll();

	    if (verbose) System.out.println("  found " + aucCollect.size() + " auctions. - pruning");
	    
	    Object[] auctionBeans= aucCollect.toArray();
	    
	    AuctionObj[] auctionObjs = (AuctionObj[])Array.newInstance(AuctionObj.class, 1);
	    ArrayList auctionObjList = new ArrayList();
	    

	    for(int i = 0; i< Array.getLength(auctionBeans); i++){
		AuctionObj temp = ((AuctionEntityLocal)auctionBeans[i]).toAuctionObj();
		if((temp.getDescription() != null &&
		    (temp.getDescription()).toLowerCase().indexOf(name.toLowerCase()) > -1 ) ||
		   (temp.getTitle() != null &&
		    (temp.getTitle()).toLowerCase().indexOf(name.toLowerCase()) > -1 )){
		    auctionObjList.add(validateAuctionData(temp));
		}
	    }
	    
	    if (verbose) System.out.println("   " + auctionObjList.size() + " auctions matching.");

	    return (AuctionObj[])auctionObjList.toArray(auctionObjs);
	    
	} catch(FinderException fe) {
	    if (verbose) System.out.println("Unable to locate auctions with search string = " + name);
	    throw new InvalidAuction("Error with search, \"" + name + "\" not found");	    
	} catch(NamingException ne) {
	    if (verbose) System.out.println("Error during JNDI lookup of beans.");
	    throw new InvalidAuction("Error with search, JNDI problem.");	 
	} catch(Exception e) {
	    e.printStackTrace();
	    throw new InvalidAuction("Error with search...?");
	}
	//	throw new java.rmi.RemoteException();
    }
    
    public void BidPost(Integer sessionKey, Integer auctionID, Double bid, Integer TID) throws java.rmi.RemoteException, InvalidAuction, InvalidBid, UserNotLoggedIn {
	
	if (verbose) System.out.println("BidPost Called");
		
	// Verify positive auction and Bid IDs
	if(auctionID.intValue() <= 0 || bid.doubleValue() < 0.0)
	    throw new InvalidBid("AuctionID and bid value must be positive");

	// Check to make sure the sessionKey corresponds to a known UserID
	try
	{
	    Context jndiCntx = new InitialContext();
	    
	    //check if this auction is already posted
	    Object Uref = jndiCntx.lookup("UserEntityLocalHome");
	    UserEntityLocalHome Uhome = (UserEntityLocalHome)PortableRemoteObject.narrow(Uref,UserEntityLocalHome.class);
	    UserEntityLocal user_1 = Uhome.findByPrimaryKey(sessionKey);
	    
	    if(user_1.getTransID().equals(TID)){
		// we need to find the bid/auction that was posted previously

		if (verbose) System.out.println("Bid already exists in database, based on transID... find it!");
		
		return;  //jtrapass TODO: THIS IS WRONG!!!
	    }else{
		
		// Create the bid and then insert timestamp from the database
		BidObj newBid = new BidObj(new Integer(1), auctionID, 
					   sessionKey, bid, "NOW"); 
	        
		// Ask the database for the current time
		ResultSet rs = DatabaseAccessor.query("SELECT NOW()");
		
		// Set the date of the bid to what the DB says or error
		if(rs.first()){	  
		    if (verbose) System.out.println("Setting date to: " + rs.getString(1));
		    newBid.setBidDate(rs.getString(1));
		}
		else{
		    if (verbose) System.out.println("Error obtaining date from database");
		    throw new RemoteException("Can't find database");
		}
		
		if (verbose) System.out.println("Attempting to create bid: " + newBid);
		
		Object ref = jndiCntx.lookup("BidEntityLocalHome");
		BidEntityLocalHome bidhome = (BidEntityLocalHome)
		    PortableRemoteObject.narrow(ref,BidEntityLocalHome.class);  
		bidhome.create(newBid, TID);
	    }
	}
	catch(IllegalArgumentException iae){
	    if (verbose) System.out.println("Could not create bid");
	    throw new InvalidBid("Could not create bid");
	}catch(NullPointerException npe){
	    if (verbose) System.out.println("Database could not read time correctly");
	    throw new InvalidBid("Database could not create bid, please contact the server administrator");
	}catch(NamingException ne){
	    if (verbose) System.out.println("Error during JNDI lookup of beans.");
	    throw new RemoteException(ne.getMessage());
	}catch(FinderException fe){
	    if (verbose) System.out.println("Unable to verify bid on auction " + auctionID);
	    throw new InvalidAuction("Unable to verify bid on auction " + auctionID);
	}catch(InvalidBid ib){
	    if (verbose) System.out.println("Unable to create the bid");
	    throw ib;
	}catch(RemoteException re){
	    throw new RemoteException(re.getMessage());
	}catch(Exception e){
	    //if (verbose) System.out.println(e);
	    throw new InvalidBid(e.getMessage());
	}
    }
    
    public BidObj[] BidHistory(int auctionID) throws java.rmi.RemoteException, InvalidAuction
    {
	if (verbose)
	    System.out.println("BidHistory Called");
        
        if (auctionID < 0)
  	    throw new InvalidAuction("AuctionID must be greater than 0");
	
	try
	{
	    Context jndiCntx = new InitialContext();
	    
	    Object ref = jndiCntx.lookup("AuctionEntityLocalHome");
	    AuctionEntityLocalHome home = (AuctionEntityLocalHome)
		PortableRemoteObject.narrow(ref,AuctionEntityLocalHome.class);   
	    AuctionEntityLocal auction = home.findByPrimaryKey(new Integer(auctionID));
	    
	    if(auction == null)
		return null;

	    return auction.getBidHistory();
	    
	}
	catch(javax.ejb.FinderException fe)  
	{
	    if (verbose) System.out.println("Unable to find bids for that auction with ID = " + auctionID);
	    throw new InvalidAuction("Auction with ID " + auctionID + " not found");
	}
	catch(javax.naming.NamingException ne)
	{
	    if (verbose) System.out.println("Error during JNDI lookup of beans.");
	    throw new InvalidAuction("Auction with ID " + auctionID + " not found, JNDI problems");
	}
	catch(Exception e)
	{
	    if (verbose) System.out.println(e);
	    throw new InvalidAuction("Auction with ID " + auctionID + " not found...?");
	}
	//       return null;
    }

    public void isAlive() throws java.rmi.RemoteException
    {}

    public void setSessionContext( SessionContext context ) {
	this.context = context;
    }
    
    private boolean verifyUser(Integer sessionKey) throws NumberFormatException, FinderException, NamingException
    {
	Context jndiCntx = new InitialContext();
	Object ref = jndiCntx.lookup("UserEntityLocalHome");
	UserEntityLocalHome home = (UserEntityLocalHome)
	    PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);
	UserEntityLocal userBean = home.findByPrimaryKey(sessionKey);
        
	if (userBean == null)
	    return false;

	return true;
    }

    /* Take an auction object and correctly set its CurrentBid and
     * HighestBidder fields since these should ultimately be removed 
     * from the database and always looked up.
     *
     * @param auc Input auction object
     *
     * @return auction object with valid CurrentBid and HighestBidder values
     */
    private AuctionObj validateAuctionData(AuctionObj auc) throws java.rmi.RemoteException, InvalidAuction, IllegalArgumentException
    {
	BidObj[] bids = BidHistory(auc.getAuctionID().intValue());
	BidObj maxBid = new BidObj(new Integer(0), new Integer(0), new Integer(0) , new Double(0), "NOW");

	if(bids != null)
	{
	    // Find the current maximum bid
	    for(int i = 0; i < bids.length; i++)
	    {
		if((bids[i].getAmount()).compareTo(maxBid.getAmount()) > 0)
		{
		    maxBid = bids[i];
		}
	    }  
	}
      
	AuctionObj newAuc = new AuctionObj(auc.getTitle(),
					   auc.getUserID(),
					   auc.getDescription(),
					   maxBid.getUserID(),
					   auc.getStartDate(),
					   auc.getEndDate(),
					   auc.getStartBid(),
					   maxBid.getAmount());
	newAuc.setAuctionID(auc.getAuctionID());
	return newAuc;
    }


    public void ejbCreate() {}
    
    protected SessionContext context;

    public void ejbActivate() {}
    public void ejbPassivate() {}
    public void ejbRemove() {}

}
