
   /**
    * AuctionEntityBean -- Base Account Bean
    * This class must extend
    * javax.ejb.SessionBean and implement
    * the methods in this interface as well as providing
    * the implementation of the business methods.
    *
    */

package ejbay.auction;
import ejbay.account.*;
import java.util.*;
import javax.ejb.*;   
import javax.ejb.EntityContext;
import javax.ejb.CreateException;
import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import java.util.Collection;

public abstract class AuctionEntityBean implements EntityBean, AuctionEntityLocal{

    public void setEntityContext( EntityContext context ) {
	this.context = context;
    }
    public void unsetEntityContext() {
	this.context = null;
    }
    public void ejbStore() {}
    public void ejbLoad() {}  
    public void ejbActivate() {}
    public void ejbPassivate() {}
    public void ejbRemove() {}
    protected EntityContext context;

    static final boolean verbose = false;

    public java.lang.Integer ejbCreate(AuctionObj auc, Integer TID) throws CreateException
    {
        this.setTitle(auc.getTitle());
        
	this.setDescription(auc.getDescription());
	this.setHighestBidder(auc.getHighestBidder());
	this.setStartDate(auc.getStartDate());
	this.setEndDate(auc.getEndDate());
	this.setCurrentBid(auc.getCurrentBid());
	this.setStartBid(auc.getStartBid());
	
	return null; // TODO is this right!?
    }
    public void ejbPostCreate(AuctionObj auc, Integer TID)throws CreateException
    {
	//set foreign keys
	//this.setUserID(auc.getUserID());
	try{
	    Context jndiCntx = new InitialContext();

	    Object ref = jndiCntx.lookup("UserEntityLocalHome");
	    UserEntityLocalHome Uhome = (UserEntityLocalHome)PortableRemoteObject.narrow(ref,UserEntityLocalHome.class);
	    UserEntityLocal myUser = Uhome.findByPrimaryKey(auc.getUserID());
	    
	    if(myUser == null)
		throw new CreateException("Invalid UserID");
	    else{
		this.setUser(myUser);
		myUser.setTransID(TID);
	    }
	}catch(NamingException ne) {
	    if (verbose) System.out.println("Error during JNDI lookup of beans.");
	    ne.printStackTrace();
	    throw new CreateException("Error during JNDI lookup");
	} catch(CreateException e) {
	    throw e;
	} catch(Exception e) {
	    e.printStackTrace();
	    throw new CreateException();
	}
    }
    
    public AuctionObj toAuctionObj(){
	
	AuctionObj auc = new AuctionObj(
				   this.getTitle(),
				   this.getUser().getUserID(),
				   this.getDescription(),
				   this.getHighestBidder(),
				   this.getStartDate(),
				   this.getEndDate(),
				   this.getStartBid(),
				   this.getCurrentBid()
				   );
	auc.setAuctionID(this.getAuctionID());
	return auc;
    };

	public abstract Integer getAuctionID();
	public abstract void setAuctionID(Integer auctionID);
	public abstract void setTitle(String title);
	public abstract String getTitle();
    //public abstract void setUserID(Integer UserID);
    //public abstract Integer getUserID();
	public abstract void setStartBid(Double startBid);
	public abstract java.lang.Double getStartBid();
	public abstract void setCurrentBid(Double currentBid);
	public abstract Double getCurrentBid();
	public abstract void setDescription(String description);
	public abstract String getDescription();
	public abstract void setHighestBidder(Integer highestBidder);
	public abstract Integer getHighestBidder();
	public abstract void setStartDate(String startDate);
	public abstract String getStartDate();
	public abstract void setEndDate(String endDate);
	public abstract String getEndDate();

    public abstract Collection getBids();
    public abstract void setBids(Collection bids);
    public abstract UserEntityLocal getUser();
    public abstract void setUser(UserEntityLocal user);

    
    
    public BidObj[] getBidHistory(){
	BidObj bidObjs[] = null;
	try
	{

	    Collection bids = getBids();

	    if(bids == null)
		return null;
	    
	    bidObjs = new BidObj[bids.size()];
	    Iterator bidObjIterator = bids.iterator();

	    int i = 0;
	    while(bidObjIterator.hasNext())
	    {
		bidObjs[i] = ((BidEntityLocal)bidObjIterator.next()).toBidObj();
		bidObjs[i].setAuctionID(getAuctionID());
		i++;
	    }
	}
	catch(Exception e)
	{
	    if (verbose) System.out.println(e);
	}
	
        return bidObjs;
    }
    
    /* will return the max bid on success, or null if there are none */
    public BidObj getMaxBid(){
	if(verbose)
	    if (verbose) System.out.println("getMaxBid called");
	
	try{
	    
	    Collection bids = getBids();
	    
	    if(bids == null)
		return null;
	    
	    BidEntityLocal highestBid = null;

	    Iterator bidObjIterator = bids.iterator();

	    if(bidObjIterator.hasNext()){
		highestBid = (BidEntityLocal)bidObjIterator.next();
		if(verbose)
		    if (verbose) System.out.println(" -- currentMax = " + highestBid.getAmount());
	    }
	    while(bidObjIterator.hasNext()){
		BidEntityLocal bid = (BidEntityLocal)bidObjIterator.next();
		
		if(bid.getAmount().compareTo(highestBid.getAmount()) > 0){
		    highestBid = bid;
		    if(verbose)
			if (verbose) System.out.println(" -- currentMax = " + highestBid.getAmount());
		}
		else{
		    if(verbose)
			if (verbose) System.out.println(" -- notMax = " + bid.getAmount());
		}
	    }
	    
	    if(highestBid == null)
		return null;
	    
	    BidObj returnMe = highestBid.toBidObj();
	    returnMe.setAuctionID(getAuctionID());

	    if(verbose)
		if (verbose) System.out.println(" -- returning = " + returnMe.getAmount());

	    return returnMe;
	}
	catch(Exception e)
	{
	    if (verbose) System.out.println(e);
	}
	
        return null;
    }
}
