package ejbay.auction;

import java.io.Serializable;
import java.lang.IllegalArgumentException;

/**
 * ideas loosly borrowed from 2005 team 1 Account.java
 * @author phs 
 */

public class AuctionObj implements java.io.Serializable {
    /**
     *Matches this sql database stuff:
     *
     *mysql> describe Auctions;
     * +---------------+--------------+------+-----+---------+----------------+
     * | Field         | Type         | Null | Key | Default | Extra          |
     * +---------------+--------------+------+-----+---------+----------------+
     * | AuctionID     | int(11)      | NO   | PRI | NULL    | auto_increment |
     * | Title         | varchar(200) | NO   |     |         |                |
     * | UserID        | int(11)      | NO   |     |         |                |
     * | StartBid      | decimal(8,2) | NO   |     |         |                |
     * | CurrentBid    | decimal(8,2) | NO   |     |         |                |
     * | Description   | text         | YES  |     | NULL    |                |
     * | HighestBidder | int(11)      | YES  |     | NULL    |                |
     * | StartDate     | datetime     | NO   |     |         |                |
     * | EndDate       | datetime     | YES  |     | NULL    |                |
     * +---------------+--------------+------+-----+---------+----------------+
     */

    /**
     * AuctionID is autogenerated by the database, so don't create it outselves,
     * however, there must be an infrastructure for accepting it since we do
     * queries, etc
     */
    private Integer AuctionID;
    private String Title;
    private Integer UserID;
    private Double StartBid;
    private Double CurrentBid;
    private String Description;
    private Integer HighestBidder;
    private String StartDate;
    private String EndDate;
    
    /**
     * base creation format
     *   
     * @param title not permitted to be null or \"\"
     * @param UserID not permitted to be null or \"\"
     * @param description not permitted to be null or \"\"
     * @param startDate not permitted to be null or \"\"
     * @param endDate not permitted to be null or \"\"
     * @param startBid not permitted to be < 0.0
     */
    public AuctionObj(String title,
                      Integer userID,
                      String description,
		      Integer highestBidder,
		      String startDate,
		      String endDate,
		      Double startBid,
		      Double currentBid) 
	throws IllegalArgumentException
    {
	/*consistancy checking --pulled, causes exceptions with multiphase processing
	if(title == null || title.equals(""))
	    throw new IllegalArgumentException("title");
	if(userID == null || userID.intValue() < 0)
	    throw new IllegalArgumentException("UserID");
	if(description == null || description.equals(""))
	    throw new IllegalArgumentException("description");
	if(startDate == null || startDate.equals(""))
	    throw new IllegalArgumentException("startDate");
	if(endDate == null || endDate.equals(""))
	    throw new IllegalArgumentException("endDate");  
	if(startBid == null || startBid.doubleValue() < 0.0)
	    throw new IllegalArgumentException("startBid");          
	*/

	//place holder for value not yet placed in database
        AuctionID = new Integer(-1);
        Title = title;
        UserID = userID;
        Description = description;
        HighestBidder = highestBidder;
        StartDate = startDate;
        EndDate = endDate;
        StartBid = startBid;
        CurrentBid = currentBid;
    }

    public AuctionObj(String title,
		      Integer userID,
		      String description,
		      Double startBid) throws IllegalArgumentException
    {
    	Title = title;
    	UserID = userID;
    	Description = description;
    	StartBid = startBid;
    	CurrentBid = new Double(0.0);
    }

    public String toString(){
	return 
	    "[AuctionID = " + AuctionID +
	    ", Title = " + Title +
	    ", UserID = " + UserID +
	    ", StartBid = " + StartBid + 
            ", CurrentBid = " + CurrentBid +                
	    ", Description = " + Description +
	    ", HighestBidder = " + HighestBidder +
	    ", StartDate = " + StartDate +
	    ", EndDate = " + EndDate +"]";
    }

    public void setAuctionID(Integer auctionID){
	AuctionID = auctionID;
    }
    public Integer getAuctionID(){
	return AuctionID;
    }
    
    public void setTitle(String title){
	Title = title;
    }
    public String getTitle(){
	return Title;
    }

    public void setUserID(Integer userID){
	UserID = userID;
    }
    public Integer getUserID(){
	return UserID;
    }

    public void setStartBid(Double startBid){
	StartBid = startBid;
    }
    public Double getStartBid(){
	return StartBid;
    }

    public void setCurrentBid(Double currentBid){
	CurrentBid = currentBid;
    }
    public Double getCurrentBid(){
	return CurrentBid;
    }
    
    public void setDescription(String description){
	Description = description;
    }
    public String getDescription(){
	return Description;
    }

    public void setHighestBidder(Integer highestBidder){
	HighestBidder = highestBidder;
    }
    public Integer getHighestBidder(){
	return HighestBidder;
    }

    public void setStartDate(String startDate){
	StartDate = startDate;
    }
    public String getStartDate(){
	return StartDate;
    }

    public void setEndDate(String endDate)
    {
	EndDate = endDate;
    }
    public String getEndDate()
    {
	return EndDate;
    }
}
