
   /**
    * BidEntityBean -- Base Bid Bean
    * This class must extend
    * javax.ejb.SessionBean and implement
    * the methods in this interface as well as providing
    * the implementation of the business methods.
    *
    */

package ejbay.auction;
import ejbay.account.*;
import ejbay.exceptions.*;
import java.util.*;
import javax.ejb.*;   
import javax.ejb.EntityContext;
import javax.ejb.CreateException;
import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import java.text.SimpleDateFormat;
import java.text.ParsePosition;
import java.net.*;

import ejbay.account.*;

public abstract class BidEntityBean implements EntityBean, BidEntityLocal{
    

    public void setEntityContext( EntityContext context ) {
	this.context = context;
    }
    public void unsetEntityContext() {
	this.context = null;
    }
    public void ejbStore() {}
    public void ejbLoad() {}  
    public void ejbActivate() {}
    public void ejbPassivate() {}
    public void ejbRemove() {}
    protected EntityContext context;

    static final boolean verbose = false;

    public java.lang.Integer ejbCreate(BidObj bid, Integer TID) throws CreateException
    {
	this.setBidID(bid.getBidID());
	this.setAmount(bid.getAmount());
	this.setBidDate(bid.getBidDate());

	return null; // TODO is this right!?
    }

    public void ejbPostCreate(BidObj bid, Integer TID)throws CreateException, InvalidBid
    {
	/*
	  look!  A useful error message: moving to postCreate
	  05:11:45,995 INFO  [STDOUT] Attempting to create bid: [BidId = 1, AuctionID = 21, UserID = 7, Amount = 105.0, BidDate = 2006-04-02 05:11:45.0]
	  05:11:45,999 ERROR [STDERR] java.lang.IllegalStateException: A CMR field cannot be set in ejbCreate; this should be done in the ejbPostCreate method instead [EJB 2.0 Spec. 10.5.2].
	 */
	//now set foreign keys:
	try{
	    Context jndiCntx = new InitialContext();
	    
	    Object Uref = jndiCntx.lookup("UserEntityLocalHome");
	    UserEntityLocalHome Uhome = (UserEntityLocalHome)PortableRemoteObject.narrow(Uref,UserEntityLocalHome.class);
	    UserEntityLocal myUser = Uhome.findByPrimaryKey(bid.getUserID());
	    
	    if(myUser == null)
		throw new CreateException("Invalid UserID");

	    Object Aref = jndiCntx.lookup("AuctionEntityLocalHome");
	    AuctionEntityLocalHome Ahome = (AuctionEntityLocalHome)PortableRemoteObject.narrow(Aref,AuctionEntityLocalHome.class);
	    AuctionEntityLocal myAuction = Ahome.findByPrimaryKey(bid.getAuctionID());
	    
	    if(myAuction == null)
		throw new CreateException("Invalid AuctionID");
	    else{
		// Check date stamps on the auction and bid
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date aucDay = df.parse(myAuction.getEndDate(), new ParsePosition(0));
		Date bidDay = df.parse(bid.getBidDate(), new ParsePosition(0));
		
		if (verbose) System.out.println("Auction Ends: " + aucDay);
		if (verbose) System.out.println("Bid occured at: " + bidDay);
		
		// If this bid's date is greater than the auction's end date error
		if(bidDay.compareTo(aucDay) > 0){
		    throw new InvalidBid("Auction already over");
		}
	    }

	    //check that we're the biggest bid
	    if(myAuction.getStartBid().compareTo(bid.getAmount()) > 0) {
		throw new InvalidBid("New Bid is less than Start Bid");
	    }	    
	    
	    BidObj maxBid = myAuction.getMaxBid();
	    
	    if((maxBid == null) || 
	       (maxBid.getAmount() == null) ||
	       (maxBid.getAmount().compareTo(bid.getAmount()) < 0)
	       )
	    {
		// Bid is high enough, so post it
		
		this.setUser(myUser);
		this.setAuction(myAuction);
		
		//Race condition, but won't hurt much... I think
		myAuction.setCurrentBid(bid.getAmount());
		myAuction.setHighestBidder(bid.getUserID());

		myUser.setTransID(TID);
	    }
	    else{
		// Bid is too low so throw exception and return
		throw new InvalidBid("Bid is too low");
	    }
	}catch(NamingException ne) {
	    if (verbose) System.out.println("Error during JNDI lookup of beans.");
	    ne.printStackTrace();
	    throw new CreateException("Error during JNDI lookup");
	} catch(CreateException e) {
	    throw e;
	} catch(Exception e) {
	    if (verbose) System.out.println("Exception when creating the bid because no auction with the given ID exists");
	    throw new InvalidBid("no auction with the given ID exists");
	}
    }
    
	public BidObj toBidObj()
	{
		BidObj bid = new BidObj(
				   this.getBidID(),
				   null,//this.getAuction().getAuctionID(),  //always called through auctionEntityBean, non reentrant
				   ((this.getUser() != null)?this.getUser().getUserID():null),
				   this.getAmount(),
				   this.getBidDate()
				   );
		return bid;
    }

	public abstract void setBidID(Integer bidID);
	public abstract Integer getBidID();
    //  public abstract void setAuctionID(Integer auctionID);
    //	public abstract Integer getAuctionID();
    //  public abstract void setUserID(Integer userID);
    //	public abstract Integer getUserID();
	public abstract void setAmount(Double userID);
	public abstract Double getAmount();
	public abstract void setBidDate(String bidDate);
	public abstract String getBidDate();

    public abstract UserEntityLocal getUser();
    public abstract void setUser(UserEntityLocal user);
    public abstract AuctionEntityLocal getAuction();
    public abstract void setAuction(AuctionEntityLocal auction);
}
