
package ejbay.clients;

import ejbay.account.AccountHomeRemote;
import ejbay.account.AccountRemote;
import ejbay.account.UserObj;
import ejbay.auction.*;
import ejbay.exceptions.*;
import ejbay.Probe.*;
import ejbay.util.*;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import java.io.*;
import java.util.Properties;
import java.lang.*;
import java.util.*;

import java.lang.reflect.Array;

public class AutoClient {
    public static void main(String [] args) {
        BufferedReader cmdLine=new BufferedReader(new InputStreamReader(System.in));
        int count = 0;
        
        time = new Time();

        // Check to make sure that the necessary input arguments are provided
        if(args.length != 7) {
            System.out.println("Error: Incorrect parameters passed");
            System.out.println(" usage: java AutoClient <JNDI server> <numClients> <replySize> <reqTime> <hostname> <clientID> <numReqs>");
            System.exit(1);
        }
        
        try
        {
            numClients  = Integer.parseInt(args[1]);
            replySize   = Integer.parseInt(args[2]);
            reqTime     = Long.parseLong(args[3]);
            hostname    = args[4];
            userID      = Integer.parseInt(args[5]);
            numReqs     = Integer.parseInt(args[6]);
        }catch(Exception e) {
            System.out.println("Error in input parameters");
        }

        // Conect to the global JNDI server that should have been passed in
        try
        {
            globalJndi = getInitialContext(args[0], GLOBAL_JNDI_PORT);
            
        }catch(javax.naming.NamingException ne){
            System.out.println("Error: Could not connect to the global JNDI server");
            System.out.println("       make sure the server is started and the\n" +
                               "       proper address was given");
            System.exit(1);
        }
        
        probe1 = new Probe("DATA749_app_out_cli_WARM_PASSIVE_2srv_" + numClients + "cli_" + reqTime
                           + "us_" + replySize + "req_" + hostname + "_team4.txt"); 
        probe2 = new Probe("DATA749_app_in_cli_WARM_PASSIVE_2srv_" + numClients + "cli_" + reqTime
                           + "us_" + replySize + "req_" + hostname + "_team4.txt"); 
        probe3 = new Probe("DATA749_app_msg_cli_WARM_PASSIVE_2srv_" + numClients + "cli_" + reqTime
                           + "us_" + replySize + "req_" + hostname + "_team4.txt"); 
        try 
        {             
            int cmd = -1;                            

            if(!initServerConnection())
            {
                System.out.println("Initial connection failed, please make sure JNDI is launced");
                System.exit(1);
            }
            
            acctLogin();
            aucFacade.clientInit(numClients, replySize, (int)reqTime, hostname);            
            while(count < numReqs)
            {                             
                try{
                    tid++;
                    if(count < numReqs/2)
                        aucPost();
                    else
                        aucView();

                    if(reqTime != 0)
                        Thread.sleep(reqTime);

                } catch (Exception e){
                    System.out.println("Auto Client failed, killing auto client"); 
                    return;
                }
                count++;
            }            
            aucFacade.clientDone();

        }catch (java.io.IOException ioe) {
            ioe.printStackTrace();       
        } 
        probe1.output();
        probe2.output();
        probe3.output();
    }
    
    public static String getTime()
    {
        return String.valueOf(time.getElapsedTime());
    }

    public static boolean initServerConnection()
    {
        String curPrimary = new String("");
               
        try
        {
            curPrimary = (String)globalJndi.lookup("server0");
        }
        catch(javax.naming.NamingException ne){
            System.out.println("Error: global JNDI has no primary server");
            return false;
        }

        try
        {
            Context jndiContext = getInitialContext(curPrimary, LOCAL_JNDI_PORT);
            Object ref = jndiContext.lookup("AccountHomeRemote");
            AccountHomeRemote home = (AccountHomeRemote)
                PortableRemoteObject.narrow(ref,AccountHomeRemote.class);
            account = home.create();
            if(account == null)
                return false;
            ref = jndiContext.lookup("AuctionHomeRemote");
            AuctionHomeRemote AucHome = (AuctionHomeRemote)
                PortableRemoteObject.narrow(ref,AuctionHomeRemote.class);
            aucFacade = AucHome.create(); 
            if(aucFacade == null)
                return false;
        } catch(javax.naming.NamingException ne) {
            System.out.println("Connecting to " + curPrimary + " failed");
            //ne.printStackTrace();
            return false;
        } catch(javax.ejb.CreateException ce) {
            System.out.println("Connecting to " + curPrimary + " failed");
            //ce.printStackTrace();
            return false;
        } catch(java.rmi.RemoteException re) {
            System.out.println("Connecting to " + curPrimary + " failed");
            return false;
        }

        System.out.println("Connected to server " + curPrimary);

        return true;
        
    }
    
    /* Get a new initial context that sets up the client to find
     * the JNDI server.
     *
     * @param jndiServer The URL of the JNDI server to connect to
     * @return A handle to the context that JNDI should use
     */
    public static Context getInitialContext(String jndiServer, int port) throws javax.naming.NamingException {
        Properties env = new Properties();
        
        if(jndiServer.indexOf(":") == -1)
            jndiServer = jndiServer + ":" + Integer.toString(port);
        
        env.setProperty(Context.INITIAL_CONTEXT_FACTORY,
                "org.jnp.interfaces.NamingContextFactory");
        env.setProperty(Context.URL_PKG_PREFIXES,
                "org.jboss.naming:org.jnp.interfaces");
        env.setProperty(Context.PROVIDER_URL, jndiServer);
        
        return new InitialContext(env);
    }            
    
    public static void aucPost() throws java.rmi.RemoteException
    {
	try {
	    /* Get user input for creating new auction */
	    
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");
	    
            String aucName = "AutoClient " + myUserID + "'s auction";
            String aucDesc = "AutoClient";
	    AuctionObj auc = new AuctionObj(aucName, myUserID, aucDesc, new Double(0));	   
            AuctionObj newAuc = null;
            int [] temp = null;

            probe1.record(getTime());
            if(replySize == 0)
                newAuc = aucFacade.AuctionPost(sessionKey, auc, new Integer(1), TID());
            else
                temp = aucFacade.AuctionPost2(sessionKey, auc, new Integer(1), TID());
            probe2.record(getTime());
            probe3.record("aucPost()");
            
            if(newAuc == null && temp == null)
                throw new java.rmi.RemoteException("broken");
	} 
	catch(NumberFormatException nfe) 
	{
	    System.out.println("Error: must enter valid number values for length and startbid.");
	}
	catch(InvalidAuction auce) 
	{
	    System.out.println("Error: " + auce.getMessage());
	}
	catch(UserNotLoggedIn unli) {
	    System.out.println("You are not logged in.");
	}
	catch(IllegalArgumentException iae)
	{
	    System.out.println("Error: must enter valid number values for length and startbid.");
	}
	return;
    }
        
    public static void aucView() throws java.rmi.RemoteException {
	
        try
	{
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");
            AuctionObj auc = null;
            int [] temp = null;

            probe1.record(getTime());
            if(replySize == 0)
                auc = aucFacade.AuctionView(24);
            else
                temp = aucFacade.AuctionView2(24);
            probe2.record(getTime());
            probe3.record("aucView()");

            if(auc == null && temp == null)
                throw new java.rmi.RemoteException("broken");

        } 
	catch(NumberFormatException nfe) 
	{
            System.out.println("Error: AuctionID must be an integer");
        } 
	catch(InvalidAuction auce) 
	{
            System.out.println("Error: " + auce.getMessage());
        }
	catch(UserNotLoggedIn loginE)
	{
	    System.out.println("You are not logged in.");
	}
        return;
    }

    public static void acctLogin() throws java.rmi.RemoteException 
    {    
        try 
        {
            String user = "autoclient" + userID;
            String pass = String.valueOf(userID);
            
            myUserID = new Integer(account.AcctLogin(user, pass));
            
            if((myUserID == null) || (myUserID.equals(new Integer(0))))
                throw new InvalidUserPass();
	    
	    sessionKey = myUserID;            
	    tid=0; //0 at login, incremented to 1 after success
            
        } catch(InvalidUserInfo iui) {
            System.out.println("Bad username or password.");
            myUserID = new Integer(0);
        } catch(InvalidUserPass iup) {
            System.out.println("Bad username or password.");
            myUserID = new Integer(0);
        } catch(javax.ejb.ObjectNotFoundException onfe) {
            System.out.println("Bad username or password.");
            myUserID = new Integer(0);
        } 
        return;
    }

        
    public static Integer TID() {
	return new Integer(tid);
    }
    
    // Client connection global variables
    static AccountRemote account = null;
    static AuctionRemote aucFacade = null;
    static Context globalJndi;
    static Integer sessionKey = new Integer("0");
    static Integer myUserID = new Integer(0);
    static int tid = -1;

    static int numClients;
    static int replySize;
    static long reqTime;
    static String hostname;
    static int numReqs;
    static int userID;
    static long startTime;

    static Time time;
    static Probe probe1;
    static Probe probe2;
    static Probe probe3;

    // Define constants for the different commands that are processed
    private static final int NUM_REQUESTS       = 10000;
    private static final int GLOBAL_JNDI_PORT   = 59080;
    private static final int LOCAL_JNDI_PORT    = 6729;
}
