
package ejbay.clients;

import ejbay.account.AccountHomeRemote;
import ejbay.account.AccountRemote;
import ejbay.account.UserObj;
import ejbay.auction.*;
import ejbay.exceptions.*;
import ejbay.Probe.*;
import ejbay.util.*;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.rmi.RemoteException;
import java.io.*;
import java.util.Properties;
import java.lang.*;
import java.util.*;

import java.lang.reflect.Array;

public class Client_1 {
    public static void main(String [] args) {
        BufferedReader cmdLine=new BufferedReader(new InputStreamReader(System.in));
        failures = 0;
        boolean isDone;
        int count = 0;
        long latencyTemp = 0;

        // Check to make sure that the necessary input arguments are provided
        if(args.length > 4 || args.length <= 0 || args.length == 2 || args.length == 3) {
            System.out.println("Error: Incorrect parameters passed");
            System.out.println(" usage: java Client_1 <JNDI server> [optional: <auto-client filename> <autoClientID> <numIterations>]");
            System.exit(1);
        }
        
        // Conect to the global JNDI server that should have been passed in
        try
        {
            globalJndi = getInitialContext(args[0], GLOBAL_JNDI_PORT);
            mirror = new JNDI_Mirror(globalJndi, 3000);
            mirror.start();
            mirror.setPriority(3);

        }catch(javax.naming.NamingException ne){
            System.out.println("Error: Could not connect to the global JNDI server");
            System.out.println("       make sure the server is started and the\n" +
                               "       proper address was given");
            System.exit(1);
        }

        if(args.length == 4)
        {
            time = new Time();
            probe1 = new Probe("cli_total_latency_" + args[1]);
            probe2 = new Probe("cli_cmd_latency" + args[1]);
            probe3 = new Probe("cli_naming_srvc_latency" + args[1]);
            auto = true;
            try
            {               
                autoID = Integer.parseInt(args[2]);
                numIterations = Integer.parseInt(args[3]);
            }catch(NumberFormatException nfe)
            {
                System.out.println("Error: the last two parameters must be valid integer");
                System.exit(1);
            }              
        }     
        
        // Connect to a local JNDI server and run the client
        try 
        {             
            int cmd = -1;
            
	    while(mirror.isAlive() && (mirror.getBeans() == null)){
                try{
		    Thread.sleep(200);
		}catch(Exception e){}
	    }

	    if(!(mirror.isAlive())){
		System.out.println("Failed to connect to servers.");
		System.exit(1);
	    }
	    
            if(!auto)
            {            
                System.out.println("Welcome!");
                printHelp();
                System.out.println("Please enter your command");
            }

            while(cmd != CMD_QUIT && failures < MAX_FAILURES && mirror.isAlive())
            {
                String cmdString;
                
                if(!auto)
                {
                    System.out.print("EJBay> ");
                    // Read input from command line
                    cmdString = cmdLine.readLine();
                    cmdString = cmdString.trim();
                    // Parse input for the request
                    cmd = parseRequest(cmdString);
                }
                else
                {
                    if(count == 0)
                        cmd = CMD_ACCTLOGIN;
                    else if(count <= numIterations)
                        cmd = CMD_ACCTVIEW;
                    else
                        cmd = CMD_QUIT;
                }
                // Handle that request
                isDone = false;

                long serverTimeTemp = 0;             
   
                if(auto) latencyTemp = getTime();                
                while(!isDone && failures < MAX_FAILURES && mirror.isAlive() && cmd != CMD_QUIT)
                {

		    beans = mirror.getBeans();
		    
		    if(beans == null){  //Middle of an update, try again...
			try{
			    Thread.sleep(300);
			}catch(Exception e){}
			continue;
		    }

		    account = beans.getAccount();
		    aucFacade = beans.getAuction();
		    
                    try{
                        long temp = 0;
			
                        if(auto) temp = getTime();
                        handleRequest(cmd);
                        if(auto) {
			    probe2.record(String.valueOf(getTime() - temp));
                            probe3.record(String.valueOf(serverTimeTemp));
                            serverTimeTemp = 0;
                        }
                        
			//only update these variables after a successful invocation
                        isDone = true;
                        failures = 0;
			tid++;
                    } catch (java.rmi.RemoteException re){
                        // Failure during communication
                        System.out.println("Connection with primary failed, attempting to failover"); 
                        isDone = false;
                        
			long temp2=0;
                        if(auto) temp2 = getTime();
                        mirror.nextServer();
                        if(auto) serverTimeTemp += getTime() - temp2;
			
                        failures++;
                    }
                }
                if(auto) probe1.record(String.valueOf(getTime()-latencyTemp));
                count++;
            }
            
	    if(mirror.isAlive())
		mirror.end();
	    
            // Shouldn't really happen due to HP design changes, but be safe
            if(failures >= MAX_FAILURES)
                System.out.println("Error: Could not find an active primary");

        }catch (java.io.IOException ioe) {
            ioe.printStackTrace();       
        } 

        if(auto)  // Save HP probe data
        {
            probe1.output();
            probe2.output();
            probe3.output();
        }
    }
    
    public static long getTime()
    {
        return time.getElapsedTime();
    }

    public static ArrayList AskUser_ArrList(String[] questions) {
        ArrayList answers = new ArrayList(questions.length);
        BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
        
        try {
            String temp;
            for (int i = 0; i < questions.length; i++) {
                System.out.println(questions[i]);
                temp = input.readLine();
		if (temp.equals("") || temp.equals(null))
                    answers.add(i, " ");
                else
                    answers.add(i, temp);
            }
        } catch (IOException ioe) {
            System.out.println("IO error trying to read your input!");
            System.exit(1);
        }
        return answers;
    }
    
    public static String[] AskUser(String[] questions) {
        String[] answers = new String[questions.length];
        BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
        
        try {
            String inputString;
            for (int i = 0; i < questions.length; i++) {
                System.out.println(questions[i]);
                inputString = input.readLine();
		if (inputString.length() == 0)
                    inputString = " ";
                answers[i] = inputString;
	    }
        } catch (IOException ioe) {
            System.out.println("IO error trying to read your input!");
            System.exit(1);
        }
        return answers;
    }
    
    public static boolean isNumber(String num){
	int i;
	char c;
	for(i = 0; i<num.length(); i++) {
	    c = num.charAt(i);
	    if(c<'0' || c>'9') {
		//System.out.println(num+" Not a number");
		return false;
	    }
	}
	return true;
    }

    public static boolean isValidExpDate(int expdate){
	int expmonth = expdate/10000;
	int expyear = expdate - expmonth*10000;

	Integer year = new Integer(2006);
	Integer month = new Integer(02);
	
	if (expmonth <= 0 && expmonth > 12){
	    System.out.println(expdate + " Invalid month");
	    return false;
	}
	if ((expyear==year.intValue() && expmonth<month.intValue()) || (expyear<year.intValue())){
	    System.out.println(expdate + " Expired CC Date");
	    return false;
	}
	return true;
    }

    /**
     * Get a new initial context that sets up the client to find
     * the JNDI server.
     *
     * @param jndiServer The URL of the JNDI server to connect to
     * @param port The default port for this connection 
     * @return A handle to the context that JNDI should use
     */
    public static Context getInitialContext(String jndiServer, int port) throws javax.naming.NamingException {
        Properties env = new Properties();
        
        if(jndiServer.indexOf(":") == -1)
            jndiServer = jndiServer + ":" + Integer.toString(port);
        
        env.setProperty(Context.INITIAL_CONTEXT_FACTORY,
                "org.jnp.interfaces.NamingContextFactory");
        env.setProperty(Context.URL_PKG_PREFIXES,
                "org.jboss.naming:org.jnp.interfaces");
        env.setProperty(Context.PROVIDER_URL, jndiServer);
        
        return new InitialContext(env);
    }
    
    /** 
     * Take in a command line request and find what request is being made
     *
     * @param req The input request
     * @return The integer command value of the request
     */
    public static int parseRequest(String req) {
        // Handle basic shell commands
        if(req.equalsIgnoreCase("help"))
            return CMD_HELP;
        if(req.equalsIgnoreCase("quit") 
	   || req.equalsIgnoreCase("exit")
	   || req.equalsIgnoreCase("q"))
            return CMD_QUIT;
        
        // Handle application commands
        //   Account Commands
        if(req.equalsIgnoreCase("acctview"))
            return CMD_ACCTVIEW;
        if(req.equalsIgnoreCase("acctcreate"))
            return CMD_ACCTCREATE;
        if(req.equalsIgnoreCase("acctlogin"))
            return CMD_ACCTLOGIN;
        if(req.equalsIgnoreCase("acctupdate"))
            return CMD_ACCTUPDATE;
        if(req.equalsIgnoreCase("acctlogout"))
            return CMD_ACCTLOGOUT;
        if(req.equalsIgnoreCase("acctstatus"))
            return CMD_ACCTSTATUS;

        //   Auction Commands
        if(req.equalsIgnoreCase("aucpost"))
            return CMD_AUCPOST;
        if(req.equalsIgnoreCase("aucview"))
            return CMD_AUCVIEW;
        if(req.equalsIgnoreCase("aucsearch"))
            return CMD_AUCSEARCH;
        if(req.equalsIgnoreCase("bidpost"))
            return CMD_AUCBIDPOST;
        if(req.equalsIgnoreCase("bidhistory"))
            return CMD_AUCBIDHISTORY;
        
        return CMD_UNKNOWN;
    }
    
    /**
     * Performs the necessary action and function call for a given command
     * message.
     *
     * @param cmd Command Message ID number to process
     */
    public static void handleRequest(int cmd) throws java.rmi.RemoteException {
        // Handle shell commands
        switch(cmd) {
            case CMD_QUIT:         // Quit
                return;
            case CMD_UNKNOWN:      // Unknown
                System.out.println("Error processing request!  Check your input and try again");
                return;
            case CMD_HELP:         // Help
                printHelp();
                return;
        }
        
        // Process Account and Auction Commands
        switch(cmd) {
            case CMD_ACCTLOGIN:     // Log in to account
                acctLogin();
                break;
            case CMD_ACCTCREATE:    // Create new account
                acctCreate();
                break;
            case CMD_ACCTLOGOUT:    // Logout of account
                acctLogout();
                break;
            case CMD_ACCTUPDATE:    // Update account
                acctUpdate();
                break;
            case CMD_ACCTVIEW:      // View account
                acctView();
                break;
            case CMD_AUCPOST:       // Post an auction
                aucPost();
                break;
            case CMD_AUCSEARCH:     // Search for an auction
                aucSearch();
                break;
            case CMD_AUCVIEW:       // View an auction
                aucView();
                break;
            case CMD_AUCBIDPOST:    // Bid on an auction
                aucBidPost();
                break;
            case CMD_AUCBIDHISTORY: // Get the bid history on an auction
                aucBidHistory();
                break;
            case CMD_ACCTSTATUS:	// Show the current account status
                acctStatus();
                break;
            default:                // Hopefully we never reach here
                System.out.println("Error processing request");
                break;
        }
    }
    
    /** 
     * Prints the help message which contains a list of all possible command that this
     * shell will process.
     */
    public static void printHelp() {
        System.out.println("List of available commands:");
        System.out.println("");
        System.out.println("acctcreate       - create an account");
        System.out.println("acctlogin        - login to an account");
        System.out.println("acctstatus       - get login status");
        System.out.println("acctlogout       - logout of this account");
        System.out.println("acctupdate       - update this account data");
        System.out.println("acctview         - view the corresponding account");
        System.out.println("aucpost          - post an auction");
        System.out.println("aucsearch        - find an auction");
        System.out.println("aucview          - view this auction");
        System.out.println("bidpost          - post a bid on the given auction");
        System.out.println("bidhistory       - get this auction's bid history");
        System.out.println("help             - display this message");
        System.out.println("quit             - exit this prompt");
	System.out.println("");
    }
    
    
    // Functions for Account actions
    
    public static void acctCreate() throws java.rmi.RemoteException {
        try {
            UserObj user;
            
            String[] q = new String[8];
            
            q[0] = "What is your email address?";
            q[1] = "What is your First Name?";
            q[2] = "What is your Last Name?";
            q[3] = "Enter password?";
            q[4] = "Retype password?";
            q[5] = "Enter Credit Card Name";
            q[6] = "Enter Credit Card Number";
            q[7] = "Enter Credit Card Expiration Date (MMYYYY)";
            
            if(failures == 0) {
                ansAL = AskUser_ArrList(q);
                while ( (((String)ansAL.get(0)).compareTo("")) == 0  ||
                        (((String)ansAL.get(1)).compareTo("")) == 0  ||
                        (((String)ansAL.get(2)).compareTo("")) == 0  ||
                        (((String)ansAL.get(3)).compareTo("")) == 0  ||
                        (((String)ansAL.get(4)).compareTo("")) == 0  ||
                        (((String)ansAL.get(3)).compareTo((String)ansAL.get(4))!=0)) {
                    if (((String)ansAL.get(3)).compareTo((String)ansAL.get(4))!=0)
                        System.out.println("Error: passwords do not match");
                    else
                        System.out.println("Error: Please fill in email address, first name, last name, AND password");
                    ansAL = AskUser_ArrList(q);
                }

                if (((((String)ansAL.get(6)).compareTo("")) == 0) ||
                    ((((String)ansAL.get(7)).compareTo("")) == 0) ||
                    (!isNumber((String)ansAL.get(7))) ||
                    (!isNumber((String)ansAL.get(7))) ||
                    (!isValidExpDate(new Integer((String)ansAL.get(7)).intValue())))
                {
                    //System.out.println("Invalid Credit Card information entered" + 
                    //		   "\nPlease update account with proper information.");
                    ansAL.set(6, "");
                    ansAL.set(7, "");
                }
            }

            user = new UserObj((String)ansAL.get(0), (String)ansAL.get(1),
			       (String)ansAL.get(2), (String)ansAL.get(3), (String)ansAL.get(5),
			       (String)ansAL.get(6), (String)ansAL.get(7));
	    
            myUserID = account.AcctCreate(user);
            
	    if(myUserID == null || myUserID.equals(new Integer(0))){
		System.out.println("Failed to create account.");
		return;
	    }
	    
	    sessionKey = myUserID;
	    
	    System.out.println("New user created with sessionKey: " + sessionKey);
	}
	catch(IllegalArgumentException iae) {
	    System.out.println("Error: Value entered for " + iae.getMessage() + " is incorrectly formated");
	} catch (InvalidUserInfo e) {
	    System.out.println("Error: Invalid User Info");
	} catch (ejbay.exceptions.DuplicateAccount da){
	    System.out.println(da.getMessage());
	}        
    }
    
    public static void acctLogin() throws java.rmi.RemoteException {
        try {
            /* Get user input for creating new auction */
                        
            if (failures == 0) {
                if(!auto)
                {
                    String[] q = new String[2];
                    q[0] = "E-Mail Address:";
                    q[1] = "Password:";
                    
                    ans = AskUser(q);
                }
                else
                {
                    ans = new String[2];
                    ans[0] = "autoclient" + autoID;
                    ans[1] = String.valueOf(autoID);
                }
            }
	    
            myUserID = new Integer(account.AcctLogin(ans[0],ans[1]));

            if((myUserID == null) || (myUserID.equals(new Integer(0))))
                throw new InvalidUserPass();
	    
	    sessionKey = myUserID;
            
            System.out.println("Logged in to account "+ ans[0] +" ("+myUserID+")");
	    tid=0; //0 at login, incremented to 1 after success
            
        } catch(InvalidUserInfo iui) {
            System.out.println("Bad username or password.");
            myUserID = new Integer(0);
        } catch(InvalidUserPass iup) {
            System.out.println("Bad username or password.");
            myUserID = new Integer(0);
        } catch(javax.ejb.ObjectNotFoundException onfe) {
            System.out.println("Bad username or password.");
            myUserID = new Integer(0);
        } 
        return;
    }
    
    public static void acctStatus() throws java.rmi.RemoteException {
        try {
            if(myUserID.equals(new Integer(0)))
                System.out.println("You are not currently logged in.");
            else
                System.out.println("Currently logged in with ID "+myUserID);
        } catch(Exception e) {
            System.out.println(e);
        }
        return;
    }
    
    public static void acctLogout() throws java.rmi.RemoteException {
        myUserID = new Integer(0);
	sessionKey = new Integer("0");
        System.out.println("You have been logged out.");
    }
    
    /* This function will prompt the user for user information.
     *    A null string will use default value.
     *
     * @param args A string.
     */
    public static void acctUpdate() throws java.rmi.RemoteException {
        try 
	{
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");
	    
	    System.out.println("Enter new values for the following fields, or just\n" +
			       "hit enter to keep your current value for a particular field\n");
            String[] q  = new String[7];
            q[0] = "What is your email address?";
            q[1] = "What is your first name?";
            q[2] = "What is your last name?";
            q[3] = "What is your password?";
            q[4] = "What is the name on your credit card?";
            q[5] = "What is your credit card number?";
            q[6] = "What is the expiration date (MMYYYY)?";
          
            if (failures == 0) {
                ans = AskUser(q);
                for (int i = 0; i < ans.length; i++) {
                    if (ans[i] == "")
                        ans[i] = null;
                }

                if ((!ans[5].equals(" ") && !isNumber(ans[5])) ||
                    (!ans[6].equals(" ") && (!isNumber(ans[6]) || 
                     !isValidExpDate(new Integer(ans[6]).intValue()))))
                {
                    throw new InvalidUserInfo("Invalid Credit Card Information");
                }
            }

            UserObj user = new UserObj(ans[0],ans[1],ans[2],ans[3],ans[4],ans[5],ans[6]);
            
            account.AcctUpdate(user, sessionKey);
	    System.out.println("Account successfully updated");
        }
	catch(IllegalArgumentException iae){
	    System.out.println("Error: " + iae.getMessage());
	}
	catch(InvalidUserInfo usre) {
            System.out.println("Error: " + usre.getMessage());
        } 
        catch(UserNotLoggedIn le) {
             System.out.println("You are not logged in.");
	}
        return;
    }
    
    /** 
     * This function will take in an email address and query the server for
     * the account associated with that email address.
     */
    public static void acctView() throws java.rmi.RemoteException 
    {
        try 
	{
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");

	    String[] q  = new String[1];
            q[0] = "Enter account email address:";

            if (failures == 0) {
                if(!auto)
                {
                    ans = AskUser(q);
                    if (ans[0] == "")
                        ans[0] = null;
                }
                else
                {
                    ans = new String[1];
                    ans[0] = "autoclient1";
                }
            }
	    
            UserObj user = account.AcctView(ans[0]);
            
            if(null == user)
                System.out.println("No user found matching: " + ans[0]);
            else if(!auto)
                System.out.println("Found("+ans[0]+"):\n" + user.toString());
        } catch(InvalidUserInfo usre) {
            System.out.println("Error: " + usre.getMessage());
        } catch(UserNotLoggedIn le) {
	    System.out.println("You are not logged in.");
        }
        return;
    }


    // Functions for Auction Actions

    public static void aucPost() throws java.rmi.RemoteException
    {
	try {
	    /* Get user input for creating new auction */
	    
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");
	    
	    String[] q = new String[4];
	    q[0] = "Please enter the Title for the auction:";
	    q[1] = "Please enter the Description for the auction:";
	    q[2] = "How many days long is this auction?";
	    q[3] = "What starting bid would you like to set?";

            if (failures == 0)
                ansAL = AskUser_ArrList(q);

	    AuctionObj auc = new AuctionObj((String)ansAL.get(0),myUserID,(String)ansAL.get(1),new Double((String)ansAL.get(3)));
	    
	    AuctionObj newAuc = aucFacade.AuctionPost(sessionKey, auc,new Integer((String)ansAL.get(2)), TID());
            
	    System.out.println("\n" + newAuc);
            System.out.println("Auction posted successfully");
	} 
	catch(NumberFormatException nfe) 
	{
	    System.out.println("Error: must enter valid number values for length and startbid.");
	}
	catch(InvalidAuction auce) 
	{
	    System.out.println("Error: " + auce.getMessage());
	}
	catch(UserNotLoggedIn unli) {
	    System.out.println("You are not logged in.");
	}
	catch(IllegalArgumentException iae)
	{
	    System.out.println("Error: must enter valid number values for length and startbid.");
	}
	return;
    }
    
    public static void aucSearch() throws java.rmi.RemoteException {
        try {
	    /* Get user input for creating new auction */

	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");

            String[] q = new String[1];
            
            q[0] = "What would you like to search for?";
            
            if (failures == 0)
                ansAL = AskUser_ArrList(q);

	    AuctionObj[] auctions = aucFacade.AuctionSearch((String)ansAL.get(0));

	    if((auctions == null) || (Array.getLength(auctions) == 0)
	       || (auctions[0] == null)){
		System.out.println("No matching auctions found");
	    }
	    else{
		System.out.println("  Found:");
		for(int i = 0; i< Array.getLength(auctions); i++){
		    System.out.println("    " + auctions[i].toString());
		}
	    }
	    
        } catch(UserNotLoggedIn unli) {
	    System.out.println("You are not logged in.");
	} catch(InvalidAuction auce) {
	    System.out.println("There was an error processing your request");
	}
    }
    
    public static void aucView() throws java.rmi.RemoteException {
	
        try
	{
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");

	    String[] q  = new String[1];
            q[0] = "Enter auction ID:";

            if(failures == 0)
                ans = AskUser(q);
	    
            AuctionObj auc = aucFacade.AuctionView((new Integer(ans[0])).intValue());
            
            if(auc == null)
                System.out.println("No auction found matching: " + ans[0]);
            else
                System.out.println("Found("+ans[0]+"):\n" + auc.toString());
        } 
	catch(NumberFormatException nfe) 
	{
            System.out.println("Error: AuctionID must be an integer");
        } 
	catch(InvalidAuction auce) 
	{
            System.out.println("Error: " + auce.getMessage());
        }
	catch(UserNotLoggedIn loginE)
	{
	    System.out.println("You are not logged in.");
	}
        return;
    }
    
    public static void aucBidPost() throws java.rmi.RemoteException 
    {

	try
	{
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");
	    
	    String[] q  = new String[2];
            q[0] = "Enter auction ID:";
	    q[1] = "Enter amount to bid:";
	    
            if (failures == 0)
                ans = AskUser(q);
	    
	    aucFacade.BidPost(sessionKey, new Integer(ans[0]), new Double(ans[1]), TID());
            System.out.println("Bid posted successfully! (Be sure to monitor this auction for future change)");  
        }
	catch(NumberFormatException nfe) {
            System.out.println("Error: AuctionID must be an integer and bid must be a double");
        }
	catch(InvalidAuction auce) {
            System.out.println("Error: " + auce.getMessage());
        }
	catch(InvalidBid ib)
	{
	    System.out.println("Error: " + ib.getMessage());
	}
	catch(UserNotLoggedIn loginE)
	{
	    System.out.println("You are not logged in.");
	} 
    }
    
    public static void aucBidHistory() throws java.rmi.RemoteException 
    {
	try
	{
	    if(myUserID.equals(new Integer(0)))
	        throw new UserNotLoggedIn("Not logged in");

	    String[] q  = new String[1];
            q[0] = "Enter auction ID:";
	    
            if (failures == 0)
                ans = AskUser(q);
	    
            BidObj[] bids = aucFacade.BidHistory((new Integer(ans[0])).intValue());
            
	    int bidCount = 0;

            if(bids != null)
		for(int i = 0; i < bids.length; i++){
		    if(bids[i] != null){
			System.out.println(bids[i].toString());
			bidCount++;
		    }
		}
	    
	    if(bidCount == 0)
		System.out.println("No bids found for auction " + ans[0]);

        } catch(NumberFormatException nfe) {
            System.out.println("Error: AuctionID must be an integer");
        } catch(InvalidAuction auce) {
            System.out.println("Error: " + auce.getMessage());
        }        
	catch(UserNotLoggedIn loginE)
	{
	    System.out.println("You are not logged in.");
	}
    }
    
    public static Integer TID() {
	return new Integer(tid);
    }
    
    // Client connection global variables
    static AccountRemote account = null;
    static AuctionRemote aucFacade = null;

    static JNDI_Mirror mirror = null;
    static ServerStruct beans = null;

    static Context globalJndi;
    static Integer sessionKey = new Integer("0");
    static Integer myUserID = new Integer(0);
    static int failures = 0;
    static String[] ans;
    static ArrayList ansAL;
    static int tid = -1;
    static boolean auto = false;
    
    static Time time;
    static Probe probe1;
    static Probe probe2;
    static Probe probe3;
    static int autoID = -1;
    static int numIterations = 0;

    // Define constants for the different commands that are processed
    private static final int CMD_UNKNOWN        = -1;
    private static final int CMD_QUIT           = 0;
    private static final int CMD_HELP           = 1;
    private static final int CMD_ACCTVIEW       = 2;
    private static final int CMD_ACCTCREATE     = 3;
    private static final int CMD_ACCTLOGIN      = 4;
    private static final int CMD_ACCTUPDATE     = 5;
    private static final int CMD_ACCTLOGOUT     = 6;
    private static final int CMD_AUCPOST        = 8;
    private static final int CMD_AUCVIEW        = 9;
    private static final int CMD_AUCSEARCH      = 10;
    private static final int CMD_AUCBIDPOST     = 11;
    private static final int CMD_AUCBIDHISTORY  = 12;
    private static final int CMD_ACCTSTATUS	= 13;

    private static final int DELAY              = 3000;
    private static final int MAX_FAILURES       = 5;
    private static final int GLOBAL_JNDI_PORT   = 59080;
    private static final int LOCAL_JNDI_PORT    = 6729;
}
