package ejbay.clients;

import ejbay.account.*;
import ejbay.auction.*;
import ejbay.exceptions.*;
import ejbay.util.*;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.Binding;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import java.util.Properties;
import java.util.ArrayList;

public class JNDI_Mirror extends Thread {

    public JNDI_Mirror(Context c, int slptime){
        globalJndi = c;
        makelist();
	sleepTime = slptime;
	index = -1;
    }
    
    public static void makelist()
    {
        Servers = new ArrayList();
	doMaintenance();
	if(Servers.size()==0){
	    System.out.println("Mirror: Failed to bind to server.");
	    running = false;
	}
    }
    
    /** 
     * Insert the server given into the list in the correct order
     * based on server number.
     *
     * @param list The list to insert into
     * @param serv The server data to be inserted
     */
    private static void insertInOrder(ArrayList list, ServerStruct serv){
	int i;
	for(i = 0; i < list.size(); i++){
	    ServerStruct temp = (ServerStruct)list.get(i);
	    if(serv.getIndex() <= temp.getIndex()){
		list.add(i, serv);
		return;
	    }
	}
        //end
        list.add(serv);
    }

    /**
     * Find the server with the specified index number in the list.
     * This index num is not the index in the array list, but the
     * actual server number.
     *
     * @param list The list to search
     * @param index_num The number of the server to find
     * @return The index in the arraylist of the server with the
     *         given index_num
     */
    private static int findIndex(ArrayList list, int index_num){
	int i;
	for(i = 0; i < list.size(); i++){
	    ServerStruct temp = (ServerStruct)list.get(i);
	    if(temp.getIndex() == index_num){
		return i;
	    }
	}
	return 0;
    }

    /**
     * Maintain the server list by checking the global JNDI
     * to see what new things have happened. Also, check whether
     * the replication manager has switched primarys recently
     */
    public static void doMaintenance(){
        ArrayList TempServers = new ArrayList();
	int thisIndex = 0;
	int thisCnt = 0;
        try{
            NamingEnumeration enum = globalJndi.listBindings("");            
            while (enum.hasMore()) {
		ServerStruct Server;
                Binding binding = (Binding) enum.next();
		if(binding.getName().equals("primaryNum")){
		    thisIndex = Integer.parseInt((String)binding.getObject());
		    //System.out.println("Got primary: " + index);
		}else if(binding.getName().equals("primaryCnt")){
		    thisCnt = ((Integer)binding.getObject()).intValue();
		    //System.out.println("Got primary: " + index);    
                }else{
		    if(binding.getObject().equals("null")){
			//do nothing
		    }
		    else{                        
			Server = FindServer((String)binding.getObject());
			String name = binding.getName();
			if((thisCnt == blackListCnt) &&
			   (name.equals(blackListed)))
			    continue;
			int myIndex = Integer.parseInt(name.replaceFirst("server", ""));
			if(Server==null){
			    Server = new ServerStruct();
			    Server.setIndex(myIndex);
                            if(setServer(Server, (String)binding.getObject()))
				insertInOrder(TempServers, Server); //else throw something
			}
			else{
			    Server.setIndex(myIndex);
			    insertInOrder(TempServers, Server);
			}
		    }
		}
	    }
	    
	    synchronized(Servers){
		thisIndex = findIndex(TempServers, thisIndex);
		Servers = TempServers; //garbage collection?
		index = thisIndex;
		if(primaryCnt != thisCnt)
		    blackListed = null;
		primaryCnt = thisCnt;
		
		if(Servers.size()==0){
		    failures++;
		}else{
		    failures = 0;
		}
	    }
        } catch (NamingException e) {
            System.out.println("Mirror: Naming Error doing maintenance: " + e.getMessage());
        } catch (Exception exception) {
            System.out.println("Mirror: Error doing maintenance: " + exception.getMessage());
            exception.printStackTrace();
        }        
    }
    
    /**
     * Return a server object with the current set of account and auction
     * beans for the client to use
     *
     * @return The server object with the current primary's info
     */
    public static ServerStruct getBeans(){       
        ServerStruct temp;

        synchronized(Servers)
        {
            temp = getPrimary();
            if(temp != null)
            {
                if(userBeans == null || !userBeans.equals(temp.getName()))
                    System.out.println("Talking to new server: " + temp.getName());

                userBeans = temp.getName();
            }
            else
                userBeans = null;

            userPrimNum = primaryCnt;
        }

	return temp;
    }
    
    /**
     * Find the current primary according to the list or 
     * return null if there is no primary
     *
     * @return Server object of the current primary or null
     *         if there is no primary
     */
    public static ServerStruct getPrimary(){
	synchronized(Servers){
	    if(Servers.size()==0)
		return null;
	    if(index < 0 || index >= Servers.size())
		index = 0;
	    ServerStruct temp = (ServerStruct)Servers.get(index);
	    if(temp.getName().equals(blackListed)){
		if(Servers.size() > 1){
		    index++;
		    if(Servers.size()<=index)
			index = 0;
		    temp = (ServerStruct)Servers.get(index);
		    if(temp.getName().equals(blackListed)){
			System.out.println("BUG BUG BUG - JOE!!!");
		    }
		    return temp;
		}
		return null;
	    }
	    return temp;
	}
    }

    /**
     * Maintain the server list every 'sleepTime' seconds
     * and make sure to quit if there are too many failures
     */
    public void run(){
	failures = 0;
        while(running){
	    try{
		Thread.sleep(sleepTime);
	    }catch(Exception e){}
            doMaintenance();
	    if(failures > 5){
		System.out.println("Mirror: Failed to bind too many times, giving up.");
		running = false;
	    }
        }
    }
     
    /**
     * Advance to the next good server, because we didn't like
     * the primary that was given out last time.
     */
    public void nextServer(){
	synchronized(Servers){
	    ServerStruct primary = null;
	    if(userBeans != null){
                // Blacklist the bad server so that we don't use it
                // again until the rep manager has updated
		blackListCnt = userPrimNum;
		blackListed = userBeans;
		System.out.println("  DeadPrimary: " + blackListed);
	    }
	    else
		blackListed = null;
	    index++;
	    if(Servers.size()<=index)
		index = 0;

            if((primary = getPrimary()) != null)
                System.out.println("  TemporaryPrimary: " + primary.getName());
	}
    }

    public static AccountRemote getAccount(String bind){
        ServerStruct Server = FindServer(bind);
        if(Server==null)
            return null;
        return Server.getAccount();
    }
   
    public static AuctionRemote getAuction(String bind){
        ServerStruct Server = FindServer(bind);
        if(Server==null)
            return null;
        return Server.getAuction(); 
    }

    public static ServerStruct FindServer(String bindstring){
        int i;
        for( i=0; i<Servers.size(); i++ ){
            if(((ServerStruct)Servers.get(i)).getName().equals(bindstring)){
		return (ServerStruct)Servers.get(i);
	    }
        }
        return null;
    }
    
    private static boolean setServer(ServerStruct Server, String srvName){
        try{
            Server.setName(srvName);
            //System.out.println("Binding: " + binding.getObject().toString());
            Context jndiContext = getInitialContext(srvName, LOCAL_JNDI_PORT);
            Object ref = jndiContext.lookup("AccountHomeRemote");
            AccountHomeRemote home = (AccountHomeRemote)PortableRemoteObject.narrow(ref,AccountHomeRemote.class);
            Server.setAccount(home.create());
            
            if(Server.getAccount() == null)
                return false;
            ref = jndiContext.lookup("AuctionHomeRemote");
            AuctionHomeRemote AucHome = (AuctionHomeRemote)PortableRemoteObject.narrow(ref,AuctionHomeRemote.class);
            Server.setAuction(AucHome.create());
            if(Server.getAuction() == null)
                return false;
        }catch(Exception exception) {
            //System.out.println("Error setting up server " + srvName);
	    //exception.printStackTrace();
            return false;
        }
	
        return true;
    }
    
    /* Get a new initial context that sets up the client to find
     * the JNDI server.
     *
     * @param jndiServer The URL of the JNDI server to connect to
     * @return A handle to the context that JNDI should use
     */
    public static Context getInitialContext(String jndiServer, int port) throws javax.naming.NamingException {
        Properties env = new Properties();
        
        if(jndiServer.indexOf(":") == -1)
            jndiServer = jndiServer + ":" + Integer.toString(port);
        
        env.setProperty(Context.INITIAL_CONTEXT_FACTORY,
                "org.jnp.interfaces.NamingContextFactory");
        env.setProperty(Context.URL_PKG_PREFIXES,
                "org.jboss.naming:org.jnp.interfaces");
        env.setProperty(Context.PROVIDER_URL, jndiServer);
        
        return new InitialContext(env);
    }

    public static void end(){
	running = false;
    }
    
    private static ArrayList Servers;
    private static Context globalJndi;
    private static boolean running = true;
    private static boolean maintaining = false;
    
    private static final int GLOBAL_JNDI_PORT   = 59080;
    private static final int LOCAL_JNDI_PORT    = 6729;

    private static String blackListed = null;
    private static int blackListCnt;
    private static int primaryCnt;
    
    private static String userBeans = null;
    private static int userPrimNum = 0;

    private static int index = 0;
    private static int sleepTime;
    
    private static int failures;
}

